@@ -1,7 +1,7 @@
 """Package for doorstop."""
 
 __project__ = 'Doorstop'
-__version__ = '0.2'
+__version__ = '0.3'
 
 CLI = 'doorstop'
 GUI = 'doorstop-gui'
@@ -9,6 +9,7 @@ VERSION = __project__ + '-' + __version__
 
 try:
     from doorstop.common import DoorstopError, DoorstopWarning, DoorstopInfo
-    from doorstop.core import Item, Document, Tree, build, report
+    from doorstop.core import Item, Document, Tree
+    from doorstop.core import build, report, find_document, find_item
 except ImportError:  # pragma: no cover, manual test
     pass
