@@ -1,8 +1,4 @@
-#!/usr/bin/env python
-
-"""
-Unit tests for the doorstop.core.item module.
-"""
+"""Unit tests for the doorstop.core.item module."""
 
 import unittest
 from unittest.mock import patch, Mock, MagicMock
@@ -29,10 +25,12 @@ text: ''
 
 
 class MockItem(MockFileObject, Item):  # pylint: disable=W0223,R0902,R0904
+
     """Mock Item class with stubbed file IO."""
 
 
 class TestItem(unittest.TestCase):  # pylint: disable=R0904
+
     """Unit tests for the Item class."""  # pylint: disable=C0103,W0212
 
     def setUp(self):
@@ -183,20 +181,70 @@ class TestItem(unittest.TestCase):  # pylint: disable=R0904
 
     def test_text(self):
         """Verify an item's text can be set and read."""
-        self.item.text = "abc "
-        self.assertIn("text: |\n  abc\n", self.item._write.call_args[0][0])
-        self.assertEqual("abc", self.item.text)
+        value = "abc "
+        text = "abc"
+        yaml = "text: |\n  abc\n"
+        self.item.text = value
+        self.assertEqual(text, self.item.text)
+        self.assertIn(yaml, self.item._write.call_args[0][0])
 
-    def test_text_sentences(self):
+    def test_text_sbd(self):
         """Verify newlines separate sentences in an item's text."""
-        self.item.text = ("A sentence. Another sentence! Hello? Hi.\n"
-                          "A new line. And another sentence.")
-        expected = ("A sentence.\n"
-                    "Another sentence!\n"
-                    "Hello?\n"
-                    "Hi.\n"
-                    "A new line.\n"
-                    "And another sentence.")
+        value = ("A sentence. Another sentence! Hello? Hi.\n"
+                 "A new line (here). And another sentence.")
+        text = ("A sentence. Another sentence! Hello? Hi. "
+                "A new line (here). And another sentence.")
+        yaml = ("text: |\n"
+                "  A sentence.\n"
+                "  Another sentence!\n"
+                "  Hello?\n"
+                "  Hi.\n"
+                "  A new line (here).\n"
+                "  And another sentence.\n")
+        self.item.text = value
+        self.assertEqual(text, self.item.text)
+        self.assertIn(yaml, self.item._write.call_args[0][0])
+
+    def test_text_ordered_list(self):
+        """Verify newlines are preserved in an ordered list."""
+        self.item.text = "A list:\n\n1. Abc\n2. Def\n"
+        expected = "A list:\n\n1. Abc\n2. Def"
+        self.assertEqual(expected, self.item.text)
+
+    def test_text_unordered_list(self):
+        """Verify newlines are preserved in an ordered list."""
+        self.item.text = "A list:\n\n- Abc\n- Def\n"
+        expected = "A list:\n\n- Abc\n- Def"
+        self.assertEqual(expected, self.item.text)
+
+    def test_text_split_numbers(self):
+        """Verify lines ending in numbers are joined correctly."""
+        self.item.text = "Split at a number: 1\n42 or punctuation.\nHere."
+        expected = "Split at a number: 1 42 or punctuation. Here."
+        self.assertEqual(expected, self.item.text)
+
+    def test_text_newlines(self):
+        """Verify newlines are preserved when deliberate."""
+        self.item.text = "Some text.\n\nNote: here.\n"
+        expected = "Some text.\n\nNote: here."
+        self.assertEqual(expected, self.item.text)
+
+    def test_text_formatting(self):
+        """Verify newlines are removed around formatting."""
+        self.item.text = "The thing\n**_SHALL_** do this.\n"
+        expected = "The thing **_SHALL_** do this."
+        self.assertEqual(expected, self.item.text)
+
+    def test_text_non_heading(self):
+        """Verify newlines are removed around non-headings."""
+        self.item.text = "break (before \n#2) symbol should not be a heading."
+        expected = "break (before #2) symbol should not be a heading."
+        self.assertEqual(expected, self.item.text)
+
+    def test_text_heading(self):
+        """Verify newlines are preserved around headings."""
+        self.item.text = "should be a heading\n\n# right here"
+        expected = "should be a heading\n\n# right here"
         self.assertEqual(expected, self.item.text)
 
     def test_ref(self):
@@ -211,6 +259,18 @@ class TestItem(unittest.TestCase):  # pylint: disable=R0904
         self.assertEqual('foobar', self.item.get('ext1'))
         self.assertEqual(['ext1'], self.item.extended)
 
+    def test_extended_wrap(self):
+        """Verify a long extended attribute is wrapped."""
+        text = "This extended attribute should be long enough to wrap."
+        self.item.set('a_very_long_extended_attr', text)
+        self.assertEqual(text, self.item.get('a_very_long_extended_attr'))
+
+    def test_extended_wrap_multi(self):
+        """Verify a long extended attribute is wrapped with newlines."""
+        text = "Another extended attribute.\n\nNote: with a note."
+        self.item.set('ext2', text)
+        self.assertEqual(text, self.item.get('ext2'))
+
     def test_extended_get_standard(self):
         """Verify extended attribute access can get standard properties."""
         active = self.item.get('active')
@@ -336,7 +396,7 @@ class TestItem(unittest.TestCase):  # pylint: disable=R0904
                           Item.new, FILES, FILES, 'REQ', '', 3, 2, (1, 2, 3))
 
     def test_valid_invalid_ref(self):
-        """Verify an invalid reference fails valid."""
+        """Verify an invalid reference fails validity."""
         with patch('doorstop.core.item.Item.find_ref',
                    Mock(side_effect=DoorstopError)):
             self.assertFalse(self.item.valid())
@@ -488,16 +548,17 @@ class TestItem(unittest.TestCase):  # pylint: disable=R0904
 
 
 class TestFormatting(unittest.TestCase):  # pylint: disable=R0904
+
     """Unit tests for text formatting in Items."""  # pylint: disable=C0103
 
     ITEM = os.path.join(FILES, 'REQ001.yml')
 
     def setUp(self):
-        with open(self.ITEM, 'rb') as infile:
+        with open(self.ITEM, 'r') as infile:
             self.backup = infile.read()
 
     def tearDown(self):
-        with open(self.ITEM, 'wb') as outfile:
+        with open(self.ITEM, 'w') as outfile:
             outfile.write(self.backup)
 
     def test_load_save(self):
@@ -505,16 +566,13 @@ class TestFormatting(unittest.TestCase):  # pylint: disable=R0904
         item = Item(self.ITEM)
         item.load()
         item.save()
-        with open(self.ITEM, 'rb') as infile:
+        with open(self.ITEM, 'r') as infile:
             text = infile.read()
-        self.assertEqual(self.backup, text)
+            self.assertEqual(self.backup, text)
 
 
 class TestModule(unittest.TestCase):  # pylint: disable=R0904
+
     """Unit tests for the doorstop.core.item module."""  # pylint: disable=C0103
 
     pass
-
-
-if __name__ == '__main__':
-    unittest.main()
