@@ -1,8 +1,4 @@
-#!/usr/bin/env python
-
-"""
-Unit tests for the doorstop.core.report module.
-"""
+"""Unit tests for the doorstop.core.report module."""
 
 import unittest
 from unittest.mock import patch, MagicMock
@@ -14,6 +10,7 @@ import shutil
 from doorstop.core import report
 from doorstop.core.vcs.mockvcs import WorkingCopy
 from doorstop.common import DoorstopError
+from doorstop import settings
 
 from doorstop.core.test import FILES, ENV, REASON
 from doorstop.core.test.test_item import MockItem
@@ -27,6 +24,7 @@ ASSERT_CONTENTS = True
 
 
 class TestModule(unittest.TestCase):  # pylint: disable=R0904
+
     """Unit tests for the doorstop.core.report module."""  # pylint: disable=C0103
 
     @classmethod
@@ -58,7 +56,7 @@ class TestModule(unittest.TestCase):  # pylint: disable=R0904
         """Verify an HTML file can be created."""
         path = os.path.join('mock', 'report.html')
         report.publish(self.document, path, '.html')
-        mock_open.assert_called_once_with(path, 'wb')
+        mock_open.assert_called_once_with(path, 'w')
         mock_lines.assert_called_once_with(self.document, '.html',
                                            ignored=None)
 
@@ -85,6 +83,19 @@ class TestModule(unittest.TestCase):  # pylint: disable=R0904
         text = ''.join(line + '\n' for line in lines)
         self.assertEqual(expected, text)
 
+    def test_lines_text_item_no_ref(self):
+        """Verify a text report can be created without checking references."""
+        self.item.ref = 'abc123'
+        self.item.heading = False
+        _check_ref = bool(settings.CHECK_REF)
+        try:
+            settings.CHECK_REF = False
+            lines = report.lines(self.item, '.txt', ignored=self.work.ignored)
+            text = ''.join(line + '\n' for line in lines)
+        finally:
+            settings.CHECK_REF = _check_ref
+        self.assertIn("Reference: 'abc123'", text)
+
     def test_lines_text_document(self):
         """Verify a text report can be created from a document."""
         path = os.path.join(FILES, 'report.txt')
@@ -93,8 +104,8 @@ class TestModule(unittest.TestCase):  # pylint: disable=R0904
         text = ''.join(line + '\n' for line in lines)
         if ASSERT_CONTENTS:
             self.assertEqual(expected, text)
-        with open(path, 'wb') as outfile:
-            outfile.write(bytes(text, 'utf-8'))
+        with open(path, 'w') as outfile:
+            outfile.write(text)
 
     def test_lines_markdown_item(self):
         """Verify a Markdown report can be created from an item."""
@@ -111,8 +122,8 @@ class TestModule(unittest.TestCase):  # pylint: disable=R0904
         text = ''.join(line + '\n' for line in lines)
         if ASSERT_CONTENTS:
             self.assertEqual(expected, text)
-        with open(path, 'wb') as outfile:
-            outfile.write(bytes(text, 'utf-8'))
+        with open(path, 'w') as outfile:
+            outfile.write(text)
 
     def test_lines_html_item(self):
         """Verify an HTML report can be created from an item."""
@@ -129,14 +140,10 @@ class TestModule(unittest.TestCase):  # pylint: disable=R0904
         text = ''.join(line + '\n' for line in lines)
         if ASSERT_CONTENTS:
             self.assertEqual(expected, text)
-        with open(path, 'wb') as outfile:
-            outfile.write(bytes(text, 'utf-8'))
+        with open(path, 'w') as outfile:
+            outfile.write(text)
 
     def test_lines_unknown(self):
         """Verify iterating an unknown format raises."""
         gen = report.lines(self.document, '.a')
         self.assertRaises(DoorstopError, list, gen)
-
-
-if __name__ == '__main__':
-    unittest.main()
