@@ -1,6 +1,4 @@
-"""
-Compiles the Doorstop document hierarchy.
-"""
+"""Compiles the Doorstop document hierarchy."""
 
 import os
 import sys
@@ -9,16 +7,19 @@ import subprocess
 import logging
 from itertools import chain
 
+from doorstop.core.base import clear_document_cache, clear_item_cache
 from doorstop.common import DoorstopError, DoorstopWarning, DoorstopInfo
 from doorstop.core.document import Document
 from doorstop.core import vcs
 
 
-class Tree(object):
+class Tree(object):  # pylint: disable=R0902
+
     """A bidirectional tree structure to store the hierarchy of documents.
 
     Although requirements link "upwards", bidirectionality simplifies
     document processing and validation.
+
     """
 
     def __init__(self, document, parent=None, root=None):
@@ -27,6 +28,9 @@ class Tree(object):
         self.parent = parent
         self.children = []
         self._vcs = None
+        self._loaded = False
+        self._item_cache = {}
+        self._document_cache = {}
 
     def __str__(self):
         # Build parent prefix string (getattr to support testing)
@@ -63,6 +67,7 @@ class Tree(object):
         @return: new Tree
 
         @raise DoorstopError: when the tree cannot be built
+
         """
         if not documents:
             return Tree(document=None, root=root)
@@ -104,6 +109,7 @@ class Tree(object):
         @param document: Document to add
 
         @raise DoorstopError: if the document cannot yet be placed
+
         """
         logging.debug("trying to add '{}'...".format(document))
         if not self.document:
@@ -148,6 +154,8 @@ class Tree(object):
 
     # actions ################################################################
 
+    @clear_document_cache
+    @clear_item_cache
     def new(self, path, prefix, sep=None, digits=None, parent=None):  # pylint: disable=R0913
         """Create a new document and add it to the tree.
 
@@ -160,6 +168,7 @@ class Tree(object):
         @return: newly created and placed Document
 
         @raise DoorstopError: if the document cannot be created
+
         """
         document = Document.new(path, self.root, prefix,
                                 sep=sep, digits=digits,
@@ -174,6 +183,7 @@ class Tree(object):
             raise
         return document
 
+    @clear_item_cache
     def add(self, prefix):
         """Add a new item to an existing document by prefix.
 
@@ -182,12 +192,14 @@ class Tree(object):
         @return: newly created Item
 
         @raise DoorstopError: if the item cannot be created
+
         """
         document = self.find_document(prefix)
         self.vcs.lock(document.config)  # prevents duplicate item IDs
         item = document.add()
         return item
 
+    @clear_item_cache
     def remove(self, identifier):
         """Remove an item from a document by ID.
 
@@ -196,6 +208,7 @@ class Tree(object):
         @return: removed Item
 
         @raise DoorstopError: if the item cannot be removed
+
         """
         for document in self:
             try:
@@ -217,6 +230,7 @@ class Tree(object):
         @return: child Item, parent Item
 
         @raise DoorstopError: if the link cannot be created
+
         """
         logging.info("linking {} to {}...".format(cid, pid))
         # Find child item
@@ -236,6 +250,7 @@ class Tree(object):
         @return: child Item, parent Item
 
         @raise DoorstopError: if the link cannot be removed
+
         """
         logging.info("unlinking '{}' from '{}'...".format(cid, pid))
         # Find child item
@@ -254,6 +269,7 @@ class Tree(object):
         @param launch: open the default text editor
 
         @raise DoorstopError: if the item cannot be found
+
         """
         logging.debug("looking for {}...".format(identifier))
         # Find item
@@ -263,7 +279,7 @@ class Tree(object):
         # Open item
         if launch:
             _open(item.path, tool=tool)
-            # TODO: force the item to reload after editing
+            # TODO: force an item reload without touching a private attribute
             item._loaded = False  # pylint: disable=W0212
         # Return the item
         return item
@@ -276,12 +292,24 @@ class Tree(object):
         @return: matching Document
 
         @raise DoorstopError: if the document cannot be found
+
         """
         logging.debug("looking for document '{}'...".format(prefix))
-        for document in self:
-            if document.prefix.lower() == prefix.lower():
-                logging.debug("found document: {}".format(document))
+        try:
+            document = self._document_cache[prefix]
+            if document:
+                logging.debug("found cached document: {}".format(document))
                 return document
+            else:
+                logging.debug("found cached unknown: {}".format(prefix))
+        except KeyError:
+            for document in self:
+                if document.prefix.lower() == prefix.lower():
+                    logging.debug("found document: {}".format(document))
+                    self._document_cache[prefix] = document
+                    return document
+            logging.debug("could not find document: {}".format(prefix))
+            self._document_cache[prefix] = None
 
         raise DoorstopError("no matching prefix: {}".format(prefix))
 
@@ -293,29 +321,46 @@ class Tree(object):
         @return: matching Item
 
         @raise DoorstopError: if the item cannot be found
+
         """
         _kind = (' ' + _kind) if _kind else _kind  # for logging messages
         logging.debug("looking for{} item '{}'...".format(_kind, identifier))
-        for document in self:
-            try:
-                item = document.find_item(identifier, _kind=_kind)
-            except DoorstopError:
-                pass  # item not found in that document
-            else:
-                logging.debug("found item: {}".format(item))
+        try:
+            item = self._item_cache[identifier]
+            if item:
+                logging.debug("found cached item: {}".format(item))
                 return item
+            else:
+                logging.debug("found cached unknown: {}".format(identifier))
+        except KeyError:
+            for document in self:
+                try:
+                    item = document.find_item(identifier, _kind=_kind)
+                except DoorstopError:
+                    pass  # item not found in that document
+                else:
+                    logging.debug("found item: {}".format(item))
+                    self._item_cache[identifier] = item
+                    return item
+            logging.debug("could not find item: {}".format(identifier))
+            self._item_cache[identifier] = None
 
         raise DoorstopError("no matching{} ID: {}".format(_kind, identifier))
 
-    def valid(self):
+    def valid(self, document_hook=None, item_hook=None):
         """Check the tree (and its documents) for validity.
 
+        @param document_hook: function to call for custom document validation
+        @param item_hook: function to call for custom item validation
+
         @return: indication that the tree is valid
+
         """
         valid = True
         logging.info("checking tree...")
         # Display all issues
-        for issue in self.issues():
+        for issue in self.issues(document_hook=document_hook,
+                                 item_hook=item_hook):
             if isinstance(issue, DoorstopInfo):
                 logging.info(issue)
             elif isinstance(issue, DoorstopWarning):
@@ -327,10 +372,14 @@ class Tree(object):
         # Return the result
         return valid
 
-    def issues(self):
+    def issues(self, document_hook=None, item_hook=None):
         """Yield all the tree's issues.
 
+        @param document_hook: function to call for custom document validation
+        @param item_hook: function to call for custom item validation
+
         @return: generator of DoorstopError, DoorstopWarning, DoorstopInfo
+
         """
         documents = list(self)
         # Check for documents
@@ -338,9 +387,31 @@ class Tree(object):
             yield DoorstopWarning("no documents")
         # Check each document
         for document in documents:
-            for issue in document.issues(tree=self):
-                # Prepend the document's prefix
-                yield type(issue)("{}: {}".format(document.prefix, issue))
+            for issue in chain(document_hook(document=document, tree=self)
+                               if document_hook else [],
+                               document.issues(tree=self,
+                                               item_hook=item_hook)):
+                # Prepend the document's prefix to yielded exceptions
+                if isinstance(issue, Exception):
+                    yield type(issue)("{}: {}".format(document.prefix, issue))
+
+    @clear_document_cache
+    @clear_item_cache
+    def load(self, reload=False):
+        """Load the tree's documents and items.
+
+        Unlike the Document and Item class, this load method is not
+        used internally. Its purpose is to force the loading of
+        content in large trees where lazy loading may be too slow.
+
+        """
+        if self._loaded and not reload:
+            return
+        logging.info("loading the tree...")
+        for document in self:
+            document.load(reload=True)
+        # Set meta attributes
+        self._loaded = True
 
 
 def _open(path, tool=None):  # pragma: no cover, integration test
@@ -371,6 +442,7 @@ def build(cwd=None, root=None):
     @return: new Tree
 
     @raise DoorstopError: when the tree cannot be built
+
     """
     documents = []
 
@@ -401,6 +473,7 @@ def _document_from_path(path, root, documents):
     @param path: path to a potential document
     @param root: path to root of working copy
     @param documents: list of Documents to append results
+
     """
     try:
         document = Document(path, root)
@@ -412,3 +485,25 @@ def _document_from_path(path, root, documents):
         else:
             logging.info("found document: {}".format(document))
             documents.append(document)
+
+# convenience functions ######################################################
+
+_TREE = None  # implicitly created tree created for convenience functions
+
+
+def find_document(prefix):
+    """Find a document without an explicitly building a tree."""
+    global _TREE  # pylint: disable=W0603
+    if _TREE is None:
+        _TREE = build()
+    document = _TREE.find_document(prefix)
+    return document
+
+
+def find_item(identifier):
+    """Find an item without an explicitly building a tree."""
+    global _TREE  # pylint: disable=W0603
+    if _TREE is None:
+        _TREE = build()
+    item = _TREE.find_item(identifier)
+    return item
